package com.muyu.common.data.standard;

import com.muyu.common.data.util.EtlUtils;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.Arrays;

/**
 * 记录模型 RecordModel
 *
 * @author LeYang
 * on 2024/5/8
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@Builder
public class RecordModel {
    //[DataModel,DataModel,DataModel]

    /**
     * 不用数组处理  1.集合由动态扩容 减少扩容次数
     * 2.初始化 数组的话一个就是一个 集合是一个的话 是16 多占15个 初始化是16
     * 指定主键
     */
    private String[] keys;

    /**
     * 主键数据
     */
    private DataModel[] keyForValues;

    /**
     * 记录数据 由多个模型组成
     */
    private DataModel[] dataModelArr=null;

    /**
     * 记录长度
     */
    private int recordLength;

    public static  RecordModel build(int dataModelLength,String[] keys){
        return  build(new DataModel[dataModelLength],keys);
    }

    public static RecordModel build(DataModel[] dataModelArr,String[] keys){
        return RecordModel.builder()
                .dataModelArr(dataModelArr)
                .recordLength(dataModelArr.length)
                .keys(keys)
                .keyForValues(
                        Arrays.stream(dataModelArr).filter(dataModel -> EtlUtils.valAsArr(keys,dataModel.getKey()))
                                .toArray(value -> new DataModel[0]))
                .build();
    }
}
