package com.muyu.source.connection.pool;

import com.alibaba.druid.pool.DruidDataSource;
import com.muyu.common.core.exception.ServiceException;
import com.muyu.common.core.utils.StringUtils;
import lombok.extern.log4j.Log4j2;

import java.util.concurrent.ConcurrentHashMap;

/**
 * 连接池上下文 ConnectionPoolContext
 *
 * @author Yangle
 * Date 2024/5/15 15:05
 */
@Log4j2
public class ConnectionPoolContext {

    private ConnectionPoolContext(){

    }

    /**
     * 进行存储所有连接的容器
     */
    private  final static ConcurrentHashMap<String, DruidDataSource> connectionPoolContext=
            new ConcurrentHashMap<>(16);

    /**
     * 新增连接池
     * @param key 键
     * @param druidDataSource 连接池
     */
    public static void setConnection(String key,DruidDataSource druidDataSource){
        if (connectionPoolContext.containsKey(key)){
            throw new ServiceException(
                    StringUtils.format("连接池key-[{}]已经存在",key)
            );
        }
        connectionPoolContext.put(key,druidDataSource);
    }

    /**
     * 通过key获取连接池
     * @param key 键
     * @return
     */
    public static DruidDataSource getConnection(String key){
        return connectionPoolContext.get(key);
    }

    /**
     * 通过key删除连接池
     * @param key
     */
    public static void removeConnection(String key){
       try( DruidDataSource druidDataSource= connectionPoolContext.remove(key)){
           druidDataSource.close();
       }catch (RuntimeException e){
           log.warn("连接池关闭异常:[{}]-[{}]",key,e.getMessage(),e);
       }
    }


}
