package com.muyu.source.connection.service;

import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.druid.pool.DruidPooledConnection;
import com.muyu.source.domain.Kvt;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Component;

import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.muyu.source.connection.pool.ConnectionPoolContext.getConnection;

/**
 * 连接池的管理 ConnectionPoolManagement
 *
 * @author Yangle
 * Date 2024/5/15 15:19
 */
@Log4j2
@Component
public class ConnectionPoolManagement {
    private final static ThreadLocal<Map<DruidPooledConnection, String>> connectionToKey
            = new ThreadLocal<>();

    public static Map<DruidPooledConnection, String> getConnectionToKey() {
        Map<DruidPooledConnection, String> dataMap = connectionToKey.get();
        if (dataMap == null) {
            synchronized (connectionToKey) {
                dataMap = new HashMap<>();
                connectionToKey.set(dataMap);
            }
        }
        return dataMap;
    }

    /**
     * 初始化
     */
    public static void init(List<Kvt> kvts) {
        //初始化连接池
        kvts.forEach(kvt -> {
            DruidDataSource druidDataSource = new DruidDataSource();
            druidDataSource.setDriverClassName("com.mysql.cj.jdbc.Driver");
            druidDataSource.setUrl("jdbc:mysql://" + kvt.getHost() + ":" + kvt.getPort() + "/" + kvt.getDatabaseName() + "?" + kvt.getConnectionParam());
            druidDataSource.setUsername(kvt.getUsername());
            druidDataSource.setPassword(kvt.getPassword());
            druidDataSource.setInitialSize(kvt.getInitNum().intValue());
            druidDataSource.setMaxActive(kvt.getMaxNum().intValue());
            druidDataSource.setMinIdle(kvt.getInitNum().intValue());
            druidDataSource.setMaxWait(kvt.getMaxWaitTime());
            druidDataSource.setTimeBetweenEvictionRunsMillis(kvt.getMaxWaitSize());
            try {
                druidDataSource.init();
                log.info("初始化成功");
            } catch (SQLException e) {
                throw new RuntimeException(e);
            }
        });
    }


    /**
     * 新增
     */
    public void createPool() {

    }

    /**
     * 获取
     */
    public static DruidPooledConnection get(Kvt kvt) {
        return get(kvt.getKey());
    }

    public static DruidPooledConnection get(String key) {
        DruidDataSource druidDataSource = getConnection(key);
        DruidPooledConnection connection = null;
        try {
            connection = druidDataSource.getConnection();
            log.info("获取链接成功:[{}]", key);
            return connection;
        } catch (SQLException e) {
            log.warn("链接获取异常:[{}]-[{}]", key, e.getMessage(), e);
            throw new RuntimeException(e);
        } finally {
            getConnectionToKey().put(connection, key);
        }
    }

    /**
     * 归还
     */
    public static void returnConnection(DruidPooledConnection connection) {
        try {
            connection.close();
        } catch (SQLException e) {
            log.warn("归还连接异常:[{}]-[{}]", getConnectionToKey().get(connection), e.getMessage(), e);
            throw new RuntimeException(e);
        } finally {
            getConnectionToKey().remove(connection);
        }
    }

    /**
     * 删除
     */
    public static void remove(DruidPooledConnection druidPooledConnection) {

    }

    public static void remove(String key) {

    }
}
