package com.muyu.engine.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.domain.Result;
import com.muyu.engine.domain.EngineVersionReq;
import com.muyu.engine.domain.RuleEngine;
import com.muyu.engine.domain.rule_engine_version.RuleEngineList;
import com.muyu.engine.domain.rule_engine_version.RuleEngineVersion;
import com.muyu.engine.mapper.RuleEngineVersionMapper;
import com.muyu.engine.service.RuleEngineService;
import com.muyu.engine.service.RuleEngineVersionService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.tools.*;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.net.URI;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 规则引擎版本业务实现层 RuleEngineVersionServiceImpl
 *
 * @author LeYang
 * on 2024/5/6
 */
@Slf4j
@Service
public class RuleEngineVersionServiceImpl extends ServiceImpl<RuleEngineVersionMapper, RuleEngineVersion> implements RuleEngineVersionService  {


    @Override
    public List<RuleEngineVersion> list(RuleEngineVersion ruleEngineVersion) {
        LambdaQueryWrapper<RuleEngineVersion> queryWrapper = new LambdaQueryWrapper<>();


        if (Objects.nonNull(ruleEngineVersion.getRuleEngineId())){
            queryWrapper.eq(RuleEngineVersion::getRuleEngineId,ruleEngineVersion.getRuleEngineId());
        }
        if (Objects.nonNull(ruleEngineVersion.getCode())){
            queryWrapper.eq(RuleEngineVersion::getCode,ruleEngineVersion.getCode());
        }
        if (Objects.nonNull(ruleEngineVersion.getName())){
            queryWrapper.eq(RuleEngineVersion::getName,ruleEngineVersion.getName());
        }
        if (Objects.nonNull(ruleEngineVersion.getVersionCode())){
            queryWrapper.eq(RuleEngineVersion::getVersionCode,ruleEngineVersion.getVersionCode());
        }
        if (Objects.nonNull(ruleEngineVersion.getWriteCode())){
            queryWrapper.eq(RuleEngineVersion::getWriteCode,ruleEngineVersion.getWriteCode());
        }
        if (Objects.nonNull(ruleEngineVersion.getStatus())){
            queryWrapper.eq(RuleEngineVersion::getStatus,ruleEngineVersion.getStatus());
        }
        if (Objects.nonNull(ruleEngineVersion.getIsActivate())){
            queryWrapper.eq(RuleEngineVersion::getIsActivate,ruleEngineVersion.getIsActivate());
        }

        return list(queryWrapper);
    }
@Autowired
private RuleEngineService ruleEngineService;
    @Override
    public RuleEngineList listResult(Integer id) {
        log.info("开始查询");
        LambdaQueryWrapper<RuleEngineVersion> queryWrapper = new LambdaQueryWrapper<>();
        //根据id 查询规则
        queryWrapper.eq(RuleEngineVersion::getRuleEngineId,id);
        List<RuleEngineVersion> list = list(queryWrapper);

        LambdaQueryWrapper<RuleEngine> wrapper = new LambdaQueryWrapper<>();
        //根据id 查询规则引擎对象
        wrapper.eq(RuleEngine::getId,id);
        List<RuleEngine> ruleEngineList = ruleEngineService.list(wrapper);
        RuleEngineList collect = ruleEngineList.stream()
                .map(ruleEngine ->
                        RuleEngineList.builder()
                                .id(ruleEngine.getId())
                                .name(ruleEngine.getName())
                                .type(ruleEngine.getType())
                                .code(ruleEngine.getCode())
                                .level(ruleEngine.getLevel())
                                .isActivate(ruleEngine.getIsActivate())
                                .status(ruleEngine.getStatus())
                                .writeCode(ruleEngine.getWriteCode())
                                .engineCode(ruleEngine.getEngineCode())
                                .ruleEngineVersionList(list)
                                .build()
                        ).collect(Collectors.toList()).get(0);


        return collect;
    }

    //全局变量
//    public static String url="D:\\work\\zglkh\\cloud-server\\muyu-modules\\muyu-rule-engine\\muyu-rule-engine-common\\src\\main\\java\\com\\muyu\\engine\\custom\\";
    @Override
    public void RuleEngineVersionAdd(RuleEngineVersion ruleEngineVersion) {
          this.save(ruleEngineVersion);
    }

    @Override
    public void updataEngineVersionStatus(RuleEngineVersion ruleEngineVersion) {

        RuleEngineVersion build = RuleEngineVersion.builder()
                .status("1")
                .codeIng(ruleEngineVersion.getCodeIng())
                .build();
        this.update(build,new LambdaQueryWrapper<RuleEngineVersion>().eq(RuleEngineVersion::getId,ruleEngineVersion.getId()));
        this.writeCodeAdd(ruleEngineVersion);
        log.info("编码保存成功");
    }


    @Override
    public Result test(EngineVersionReq engineVersionReq) {
        if (engineVersionReq.getLevel().equals("data-field")){

        }
        if (engineVersionReq.getLevel().equals("data-set")){

        }
        if (engineVersionReq.getLevel().equals("recording")){

        }




        return null;
    }

//    //生产源文件
//    public void writeCodeAdd(RuleEngineVersion ruleEngineVersion) {
//        try{
//            //文件路径
//           String fileName =ruleEngineVersion.getVersionCode()+".java";
//           String filePath = url+fileName;
//
//           //编写java文件
//            ProcessBuilder pbCompiler = new ProcessBuilder("javac", fileName);
//            pbCompiler.directory(Paths.get(url).toFile());
//            pbCompiler.redirectErrorStream(true);
//            System.out.println("Writing to file: " + filePath);
//            //将字符串写入文件中
//            Path path = Paths.get(filePath);            Files.write(path,ruleEngineVersion.getCodeIng().getBytes(StandardCharsets.UTF_8));
//
//
//            // 编译
//                Process process = pbCompiler.start();
//
//                BufferedReader reader = new BufferedReader(new InputStreamReader(process.getInputStream()));
//            // 等待编译结果
//                int exitCode = process.waitFor();
//
//                if (exitCode == 0) {
//                    System.out.println("编译成功");
//                } else {
//                    System.out.println("编译失败");
//                }
//
//                //编译成class文件
//            ProcessBuilder processBuilder = new ProcessBuilder("java", ruleEngineVersion.getWriteCode());
//               processBuilder.directory(Paths.get(url).toFile());
//               processBuilder.redirectErrorStream(true);
//               Process Runprocess = processBuilder.start();
//            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(Runprocess.getInputStream()));
//            String str ;
//            while ((str = bufferedReader.readLine()) != null) {
//                System.out.println(str);
//                break;
//            }
//            Runprocess.waitFor();
//            int i = Runprocess.exitValue();
//            if (i==0){
//                System.out.println("运行成功");
//            }else {
//                System.out.println("运行失败");
//            }
//        }catch (Exception e){
//             throw new RuntimeException();
//        }
//
//    }



    private String url ="D:\\work\\zglkh\\cloud-server\\muyu-modules\\muyu-rule-engine\\muyu-rule-engine-common\\src\\main\\java\\com\\muyu\\engine\\java\\";
    private String url1 ="D:\\work\\zglkh\\cloud-server\\muyu-modules\\muyu-rule-engine\\muyu-rule-engine-common\\target\\classes\\java\\";

    //生产源文件
    public void writeCodeAdd(RuleEngineVersion ruleEngineVersion) {
        try {
            String originalVersionCode = ruleEngineVersion.getVersionCode();
            // 提取第一个字符并转换为大写
            String firstCharUpperCase = originalVersionCode.substring(0, 1).toUpperCase();
            // 截取从第二个字符到末尾的子串
            String restOfVersionCode = originalVersionCode.substring(1);
            // 将首字母大写和剩余部分拼接起来
            String newVersionCode = firstCharUpperCase + restOfVersionCode;

            String className = newVersionCode;

            //编译
            String path=url+className+".java";

            BufferedWriter bufferedWriter = new BufferedWriter(new FileWriter(path));
            // 写入内容
            bufferedWriter.write(ruleEngineVersion.getCodeIng());
            // 关闭流
            bufferedWriter.close();
            //编译内容
            String content = ruleEngineVersion.getCodeIng();
            // 指定输出目录
            JavaCompiler compiler = ToolProvider.getSystemJavaCompiler();
           // 获取编译器
            try (StandardJavaFileManager fileManager = compiler.getStandardFileManager(null, null, null)) {
                // 创建一个内存中的源文件
                JavaFileObject sourceFileObject = new JavaSourceFromString(className, content);

                // 编译选项
                Iterable<String> options = Arrays.asList("-d", url);
                // 编译源代码
                JavaCompiler.CompilationTask task = compiler.getTask(null, fileManager, null, options, null, Arrays.asList(sourceFileObject));
                boolean success = task.call();

                if (success) {
                    System.out.println("编译成功");
                } else {
                    System.out.println("编译失败");
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }catch (Exception e) {
            e.printStackTrace();
        }

    }
    //内存中的源文件
    static class JavaSourceFromString extends SimpleJavaFileObject {
        final String code;
        // 构造方法
        JavaSourceFromString(String name, String code) {
            // URI
            super(URI.create("string:///" + name.replace('.', '/') + Kind.SOURCE.extension), Kind.SOURCE);
            this.code = code;
        }
        // 返回源代码
        @Override
        public CharSequence getCharContent(boolean ignoreEncodingErrors) {
            return code;
        }
    }











//
//    private String url ="D:\\work\\zglkh\\cloud-server\\muyu-modules\\muyu-rule-engine\\muyu-rule-engine-common\\src\\main\\java\\com\\muyu\\engine\\java";
//    private String url1 ="D:\\work\\zglkh\\cloud-server\\muyu-modules\\muyu-rule-engine\\muyu-rule-engine-common\\target\\classes";
//    // 验证类名，防止路径遍历
//    // 生产源文件
//    public void writeCodeAdd(RuleEngineVersion ruleEngineVersion) {
//        try {
//
//            String className = newVersionCode + ".java";
//            String content = ruleEngineVersion.getCodeIng();
//            JavaCompiler compiler = ToolProvider.getSystemJavaCompiler();
//            StandardJavaFileManager fileManager = compiler.getStandardFileManager(null, null, null);
//            try {
//                // 写入文件
//                Path filePath = Paths.get(url, className);
//                Files.write(filePath, content.getBytes(StandardCharsets.UTF_8));
//                // 编译
//                Iterable<String> options = Arrays.asList("-d", url);
//                JavaFileObject sourceFileObject = new JavaSourceFromString(className, content);
//                JavaCompiler.CompilationTask task = compiler.getTask(null, fileManager, null, options, null, Arrays.asList(sourceFileObject));
//                boolean success = task.call();
//                log.info("编译结果: " + (success ? "成功" : "失败"));
//            } catch (Exception e) {
//                log.error("编译过程中发生错误", e);
//            } finally {
//                fileManager.close();
//            }
//        } catch (Exception e) {
//            log.error("写入代码或处理过程中发生错误", e);
//        }
//    }










}
